/***********************************
 * Author: Matt Christian
 * 
 * Description: Creates an on-screen 2D HUD
 * 
 * Notes: For use with XNA Development Tutorials
 * 
 * If you use this for a project, please give credit!
 * http://matt.insidegamer.org/xnatutorials.aspx
 * 
 * ***********************************/

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace WindowsGame1
{
    class GameSprite
    {
        //New sprite Struct
        public struct sprite
        {
            public string resourceName;
            public Vector2 screenPos;
            public Texture2D spriteTex;
            public float rotation;
            public float scale;
        };

        //Game Batch
        private SpriteBatch gameBatch;

        //List of game sprites
        public List<sprite> gameSprites = new List<sprite>();

        //Game content manager
        private ContentManager gameContent;

        /// <summary>
        /// GameSprite constructor
        /// </summary>
        /// <param name="content"></param>
        /// <param name="batch"></param>
        public GameSprite(ContentManager content, SpriteBatch batch)
        {
            //Set game content manager and create new game batch
            gameContent = content;
            gameBatch = batch;
        }

        /// <summary>
        /// GameSprite destructor
        /// </summary>
        ~GameSprite()
        {
            gameSprites.Clear();
        }

        /// <summary>
        /// Add a new sprite specifying all new values
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="spriteName"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="rotation"></param>
        /// <param name="scale"></param>
        public void AddSprite(string handle, string spriteName, float x, float y, float rotation, float scale)
        {
            //Create a new sprite
            sprite newSprite = new sprite();
            newSprite.resourceName = handle;
            newSprite.spriteTex = gameContent.Load<Texture2D>("Sprites\\" + spriteName);
            newSprite.screenPos.X = x;
            newSprite.screenPos.Y = y;
            newSprite.rotation = rotation;
            newSprite.scale = scale;

            //Add new sprite to list
            gameSprites.Add(newSprite);
        }

        /// <summary>
        /// Update sprite with all new fields
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="spriteName"></param>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="rotation"></param>
        /// <param name="scale"></param>
        public void UpdateSprite(string handle, string spriteName, float x, float y, float rotation, float scale)
        {
            //Default variables
            int position = 0;
            bool found = false;

            //See if the sprite exists
            for (int i = 0; i < gameSprites.Count; i++)
            {
                if (gameSprites[i].resourceName == handle)
                {
                    //Found the sprite
                    found = true;
                    position = i;
                }
            }

            //Can't update if we don't have a currently existing one!
            if (found)
            {
                //Create new sprite based on values
                sprite newSprite = new sprite();
                newSprite.resourceName = handle;
                newSprite.spriteTex = gameContent.Load<Texture2D>("Sprites\\" + spriteName);
                newSprite.screenPos.X = x;
                newSprite.screenPos.Y = y;
                newSprite.rotation = rotation;
                newSprite.scale = scale;

                //Swap new sprite into sprite position we want to update
                gameSprites[position] = newSprite;
            }
        }

        /// <summary>
        /// Update sprite rotation based on handle
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="spriteName"></param>
        /// <param name="rotation"></param>
        public void UpdateSpriteRotation(string handle, string spriteName, float rotation)
        {
            //Default variables
            int position = 0;
            bool found = false;

            //Look for sprite in list
            for (int i = 0; i < gameSprites.Count; i++)
            {
                if (gameSprites[i].resourceName == handle)
                {
                    //Found sprite!
                    found = true;
                    position = i;
                }
            }

            //Can't update without an old sprite
            if (found)
            {
                //Create a new sprite and old sprite holder
                sprite newSprite = new sprite();
                sprite oldSprite = new sprite();

                //Store what we want to update
                oldSprite = gameSprites[position];

                //Set values of new sprite
                newSprite.resourceName = handle;
                newSprite.spriteTex = gameContent.Load<Texture2D>("Sprites\\" + spriteName);
                newSprite.screenPos.X = oldSprite.screenPos.X;
                newSprite.screenPos.Y = oldSprite.screenPos.Y;
                newSprite.rotation = rotation;
                newSprite.scale = oldSprite.scale;

                //Swap in new sprite
                gameSprites[position] = newSprite;
            }
        }

        /// <summary>
        /// Update sprite scale based on handle
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="spriteName"></param>
        /// <param name="scale"></param>
        public void UpdateSpriteScale(string handle, string spriteName, float scale)
        {
            //Default variables
            int position = 0;
            bool found = false;

            //Find sprite
            for (int i = 0; i < gameSprites.Count; i++)
            {
                if (gameSprites[i].resourceName == handle)
                {
                    //Found sprite!
                    found = true;
                    position = i;
                }
            }

            //Can't swap if we don't have an old one
            if (found)
            {
                //Create a new sprite and old holder
                sprite newSprite = new sprite();
                sprite oldSprite = new sprite();

                //Store old sprite
                oldSprite = gameSprites[position];

                //Fill out new sprite
                newSprite.resourceName = handle;
                newSprite.spriteTex = gameContent.Load<Texture2D>("Sprites\\" + spriteName);
                newSprite.screenPos.X = oldSprite.screenPos.X;
                newSprite.screenPos.Y = oldSprite.screenPos.Y;
                newSprite.rotation = oldSprite.rotation;
                newSprite.scale = scale;

                //Swap in new sprite
                gameSprites[position] = newSprite;
            }
        }

        /// <summary>
        /// Draw all sprites
        /// </summary>
        public void DrawSprites()
        {
            //Begin our gameBatch using properties
            gameBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);
            for (int i = 0; i < gameSprites.Count; i++)
            {
                //Draw each item on the list
                gameBatch.Draw(gameSprites[i].spriteTex, gameSprites[i].screenPos, null, Color.White, gameSprites[i].rotation, new Vector2(gameSprites[i].spriteTex.Width / 2, gameSprites[i].spriteTex.Height / 2), gameSprites[i].scale, SpriteEffects.None, 0.0f);
            }

            //Finish game batch code, end
            gameBatch.End();
        }
        
        /// <summary>
        /// Clear sprite list
        /// </summary>
        public void ClearList()
        {
            gameSprites.Clear();
        }

        /// <summary>
        /// Remove sprite based on handle
        /// </summary>
        /// <param name="handle"></param>
        public void RemoveSprite(string handle)
        {
            //Default variables
            int position = 0;
            bool found = false;

            //Find sprite
            for (int i = 0; i < gameSprites.Count; i++)
            {
                if (gameSprites[i].resourceName == handle)
                {
                    //Found sprite!
                    found = true;
                    position = i;
                }
            }

            //Can't swap if we don't have an old one
            if (found)
            {
                gameSprites.RemoveAt(position);
            }
        }

    }
}
